﻿using SharpAvi;
using System;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Timers;
using System.Windows.Forms;

namespace CRecord
{
    public partial class Form1 : Form
    {
        private bool framerateOk = true;
        private bool qualityOk = true;
        private bool recording = false;
        private Recorder recorder = null;
        private volatile bool running = true;
        private Thread gc = null;
        private volatile int GC_RATE = 500;
        private volatile int PREVIEW_FPS = 10;

        public Form1()
        {
            InitializeComponent();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            framerateOk = int.TryParse(textBox1.Text, out int i);
            if (framerateOk)
                trackBar1.Value = i;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (recording)
            {
                recorder.Dispose();
                recorder = null;
                button1.Text = "Start";
            }
            else
            {
                if (!framerateOk || int.Parse(textBox1.Text) > 120)
                {
                    MessageBox.Show("Please enter the framerate correctly!");
                    return;
                }
                if (!qualityOk || int.Parse(textBox3.Text) > 100)
                {
                    MessageBox.Show("Please enter the quality correctly!");
                    return;
                }
                recorder = new Recorder(new RecorderParams(textBox2.Text, int.Parse(textBox1.Text), listBox1.SelectedIndex == 0 ? KnownFourCCs.Codecs.X264 : listBox1.SelectedIndex == 1 ? KnownFourCCs.Codecs.MotionJpeg : KnownFourCCs.Codecs.Uncompressed, int.Parse(textBox3.Text)));
                recorder.Start();
            }
            recording = !recording;
        }

        private void textBox3_TextChanged(object sender, EventArgs e)
        {
            qualityOk = int.TryParse(textBox3.Text, out int i);
            if (qualityOk)
                trackBar2.Value = i;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            gc = new Thread(() =>
            {
                while (running)
                {
                    Thread.Sleep(GC_RATE);
                    GC.Collect();
                }
            });
            gc.Start();

            var screenTimer = new System.Timers.Timer();
            screenTimer.Interval = 1000 / PREVIEW_FPS;
            screenTimer.Elapsed += new ElapsedEventHandler((object s, ElapsedEventArgs args) =>
            {
                screenTimer.Interval = 1000 / PREVIEW_FPS;
                pictureBox1.Image = Util.Screenshot();
            });
            screenTimer.Start();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            running = false;
            if (recording)
            {
                recorder.Dispose();
                recorder = null;
            }
            gc.Join();
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            textBox1.Text = trackBar1.Value.ToString();
        }

        private void trackBar2_Scroll(object sender, EventArgs e)
        {
            textBox3.Text = trackBar2.Value.ToString();
        }

        private void trackBar3_Scroll(object sender, EventArgs e)
        {
            textBox4.Text = trackBar3.Value+"ms";
            GC_RATE = trackBar3.Value;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog();
            sfd.Filter = "Video Files|*.mp4;*.mov;*.avi";
            if (sfd.ShowDialog().Equals(DialogResult.OK))
                textBox2.Text = sfd.FileName;
        }

        private void trackBar4_Scroll(object sender, EventArgs e)
        {
            textBox5.Text = trackBar4.Value.ToString();
            PREVIEW_FPS = trackBar4.Value;
        }
    }
}
